@@ -18,6 +18,9 @@
 #============================================================================
 # Copyright (C) 2006 Ivan Kelly <ivan@ivankelly.net>
 #============================================================================
+"""
+Define the CLI interface for pwman3 and the helper functions
+"""
 
 import pwman
 import pwman.exchange.importer as importer
@@ -25,8 +28,9 @@ import pwman.exchange.exporter as exporter
 import pwman.util.generator as generator
 from pwman.data.nodes import Node
 from pwman.data.tags import Tag
-from pwman.util.crypto import CryptoEngine, CryptoBadKeyException, \
-     CryptoPasswordMismatchException
+from pwman.util.crypto import CryptoEngine
+#, CryptoBadKeyException, \
+#     CryptoPasswordMismatchException
 from pwman.util.callback import Callback
 import pwman.util.config as config
 import re
@@ -35,10 +39,11 @@ import os
 import struct
 import getpass
 import cmd
-import traceback
+#import traceback
 import time
 import select as uselect
 import subprocess as sp
+import ast
 
 if sys.platform != 'win32':
     import tty
@@ -85,23 +90,23 @@ class PwmanCli(cmd.Cmd):
 
     def do_exit(self, args):
         print
-        try:
-            print "goodbye"
-            self._db.close()
-        except Exception, e:
-            self.error(e)
+        #try:
+        #    print "goodbye"
+        self._db.close()
+        #except DatabaseException, e:
+        #    self.error(e)
         return True
 
     def get_ids(self, args):
         ids = []
-        rx =re.compile(r"^(\d+)-(\d+)$")
+        rex = re.compile(r"^(\d+)-(\d+)$")
         idstrs = args.split()
         for i in idstrs:
-            m = rx.match(i)
+            m = rex.match(i)
             if m == None:
                 try:
                     ids.append(int(i))
-                except ValueError, e:
+                except ValueError:
                     self._db.clearfilter()
                     self._db.filter([Tag(i)])
                     ids += self._db.listnodes()
@@ -116,33 +121,44 @@ class PwmanCli(cmd.Cmd):
     def get_username(self, default=""):
         return getinput("Username: ", default)
 
-    def get_password(self, default=""):
-        password = getpassword("Password (Blank to generate): ", _defaultwidth,\
+    def get_password(self, argsgiven, numerics=False,leetify=False, symbols=False,
+                     special_signs=False):
+        """
+        in the config file:
+        numerics -> numerics
+        leetify -> symbols
+        special_chars -> special_signs
+        """        
+        if argsgiven == 1:
+            length = getinput("Password length (default 7): ", "7")
+            length = int(length)
+            (password, dumpme) = generator.generate_password(length, length, \
+                True, leetify, numerics, special_signs)
+            print "New password: %s" % (password)
+            return password
+        # no args given
+        password = getpassword("Password (Blank to generate): ", _defaultwidth, \
             False)
         if len(password) == 0:
             length = getinput("Password length (default 7): ", "7")
             length = int(length)
-
-            numerics = config.get_value("Generator", "numerics") == 'true'
-            leetify = config.get_value("Generator", "leetify") == 'true'
+        
             (password, dumpme) = generator.generate_password(length, length, \
-                True, leetify, numerics)
-
+                True, leetify, numerics, special_signs)
             print "New password: %s" % (password)
-            return password
-        else:
-            return password
-
+        
+        return password
+        
     def get_url(self, default=""):
         return getinput("Url: ", default)
 
     def get_notes(self, default=""):
         return getinput("Notes: ", default)
 
-    def get_tags(self, default=[]):
+    def get_tags(self, default=None):
         defaultstr = ''
 
-        if len(default) > 0:
+        if default:
             for t in default:
                 defaultstr += "%s " % (t.get_name())
         else:
@@ -184,14 +200,14 @@ class PwmanCli(cmd.Cmd):
                                     node.get_notes())
         print typeset("Tags: ", ANSI.Red),
         for t in node.get_tags():
-            print "rr %s " % t.get_name(),
+            print " %s " % t.get_name(),
         print
 
         def heardEnter():
-            i,o,e = uselect.select([sys.stdin],[],[],0.0001)
+            i, o, e = uselect.select([sys.stdin], [], [], 0.0001)
             for s in i:
                 if s == sys.stdin:
-                    input = sys.stdin.readline()
+                    sys.stdin.readline()
                     return True
                 return False
 
@@ -204,7 +220,7 @@ class PwmanCli(cmd.Cmd):
                     return True
             return False
 
-        def waituntil_enter(somepredicate,timeout, period=0.25):
+        def waituntil_enter(somepredicate, timeout, period=0.25):
             mustend = time.time() + timeout
             while time.time() < mustend:
                 cond = somepredicate()
@@ -212,13 +228,15 @@ class PwmanCli(cmd.Cmd):
                     break
                 time.sleep(period)
             self.do_cls('')
-
-        if sys.platform != 'win32':
-            print "Type Enter to flush screen (autoflash in 5 sec.)"
-            waituntil_enter(heardEnter, 5)
-        else:
-            print "Press any key to flush screen (autoflash in 5 sec.)"
-            waituntil_enter(heardEnterWin, 5)
+        
+        flushtimeout = int(config.get_value("Global", "cls_timeout"))
+        if flushtimeout > 0:
+            if sys.platform != 'win32':
+                print "Type Enter to flush screen (autoflash in 5 sec.)"
+                waituntil_enter(heardEnter, flushtimeout)
+            else:
+                print "Press any key to flush screen (autoflash in 5 sec.)"
+                waituntil_enter(heardEnterWin, flushtimeout)
 
     def do_tags(self, arg):
         tags = self._db.listtags()
@@ -309,15 +327,15 @@ class PwmanCli(cmd.Cmd):
             args = arg.split()
             if len(args) == 0:
                 types = importer.Importer.types()
-                type = select("Select filetype:", types)
-                imp = importer.Importer.get(type)
-                file = getinput("Select file:")
-                imp.import_data(self._db, file)
+                intype = select("Select filetype:", types)
+                imp = importer.Importer.get(intype)
+                infile = getinput("Select file:")
+                imp.import_data(self._db, infile)
             else:
                 for i in args:
                     types = importer.Importer.types()
-                    type = select("Select filetype:", types)
-                    imp = importer.Importer.get(type)
+                    intype = select("Select filetype:", types)
+                    imp = importer.Importer.get(intype)
                     imp.import_data(self._db, i)
         except Exception, e:
             self.error(e)
@@ -327,14 +345,14 @@ class PwmanCli(cmd.Cmd):
             nodes = self.get_ids(arg)
 
             types = exporter.Exporter.types()
-            type = select("Select filetype:", types)
-            exp = exporter.Exporter.get(type)
-            file = getinput("Select output file:")
+            ftype = select("Select filetype:", types)
+            exp = exporter.Exporter.get(ftype)
+            out_file = getinput("Select output file:")
             if len(nodes) > 0:
                 b = getyesno("Export nodes %s?" % (nodes), True)
                 if not b:
                     return
-                exp.export_data(self._db, file, nodes)
+                exp.export_data(self._db, out_file, nodes)
             else:
                 nodes = self._db.listnodes()
                 tags = self._db.currenttags()
@@ -347,7 +365,7 @@ class PwmanCli(cmd.Cmd):
                 b = getyesno("Export all nodes%s?" % (tagstr), True)
                 if not b:
                     return
-                exp.export_data(self._db, file, nodes)
+                exp.export_data(self._db, out_file, nodes)
             print "Data exported."
         except Exception, e:
             self.error(e)
@@ -358,10 +376,35 @@ class PwmanCli(cmd.Cmd):
     def do_n(self, arg):
         self.do_new(arg)
 
-    def do_new(self, arg):
+    def do_new(self, args):
+        """
+        can override default config settings the following way:
+        Pwman3 0.2.1 (c) visit: http://github.com/pwman3/pwman3
+        pwman> n {'leetify':False, 'numerics':True, 'special_chars':True}
+        Password (Blank to generate):
+        """
+        errmsg = """could not parse config override, please input some"""\
+                 +""" kind of dictionary, e.g.: n {'leetify':False, """\
+                 +"""'numerics':True, 'special_chars':True}"""
         try:
             username = self.get_username()
-            password = self.get_password()
+            if args:
+                try:
+                    args = ast.literal_eval(args)
+                except Exception:
+                    raise Exception(errmsg)
+                if not isinstance(args, dict):
+                    raise Exception(errmsg)
+                password = self.get_password(1, **args)
+            else:
+                numerics = config.get_value("Generator", "numerics").lower() == 'true'
+                # TODO: allow custom leetifying through the config
+                leetify = config.get_value("Generator", "leetify").lower() == 'true' 
+                special_chars = config.get_value("Generator", "special_chars").lower() == 'true' 
+                password = self.get_password(0,
+                                             numerics=numerics,
+                                             symbols=leetify,
+                                             special_signs=special_chars)
             url = self.get_url()
             notes = self.get_notes()
             node = Node(username, password, url, notes)
@@ -414,20 +457,20 @@ class PwmanCli(cmd.Cmd):
             if sys.platform != 'win32':
                 rows, cols = gettermsize()
             else:
-                rows,cols = 18, 80 # fix this !
+                rows, cols = 18, 80 # fix this !
             nodeids = self._db.listnodes()
             nodes = self._db.getnodes(nodeids)
             cols -= 8
             i = 0
             for n in nodes:
-                tags=n.get_tags()
+                tags = n.get_tags()
                 tagstring = ''
                 first = True
                 for t in tags:
                     if not first:
                         tagstring += ", "
                     else:
-                        first=False
+                        first = False
                     tagstring += t.get_name()
                 name = "%s@%s" % (n.get_username(), n.get_url())
 
@@ -455,7 +498,7 @@ class PwmanCli(cmd.Cmd):
         try:
             enc = CryptoEngine.get()
             enc.forget()
-        except Exception,e:
+        except Exception, e:
             self.error(e)
 
     def do_passwd(self, args):
@@ -505,19 +548,19 @@ class PwmanCli(cmd.Cmd):
         except Exception, e:
             self.error(e)
 
-    def do_cls(self,args):
+    def do_cls(self, args):
         os.system('clear')
 
-    def do_copy(self,args):
+    def do_copy(self, args):
         if self.hasxsel:
-            ids= self.get_ids(args)
+            ids = self.get_ids(args)
             if len(ids) > 1:
                 print "Can copy only 1 password at a time..."
                 return None
             try:
                 node = self._db.getnodes(ids)
                 text_to_clipboards(node[0].get_password())
-                print """copied password for %s@%s clipboard... erasing in 10 sec...""" %\
+                print "copied password for %s@%s clipboard... erasing in 10 sec..." % \
                 (node[0].get_username(), node[0].get_url())
                 time.sleep(10)
                 text_to_clipboards("")
@@ -526,7 +569,7 @@ class PwmanCli(cmd.Cmd):
         else:
             print "Can't copy to clipboard, no xsel found in the system!"
 
-    def do_cp(self,args):
+    def do_cp(self, args):
         self.do_copy(args)
 
     def do_open(self, args):
@@ -568,9 +611,6 @@ the url must contain http:// or https://."
         self.usage("cls")
         print "Clear the Screen from information."
 
-    def help_ls(self):
-        self.help_list()
-
     def help_list(self):
         self.usage("list <tag> ...")
         print "List nodes that match current or specified filter. ls is an alias."
@@ -618,7 +658,9 @@ the url must contain http:// or https://."
 
     def help_new(self):
         self.usage("new")
-        print "Creates a new node."
+        print """Creates a new node., 
+You can override default config settings the following way:      
+pwman> n {'leetify':False, 'numerics':True}"""
 
     def help_rm(self):
         self.help_delete()
@@ -643,9 +685,6 @@ the url must contain http:// or https://."
         self.usage("set [configoption] [value]")
         print "Sets a configuration option. If no value is specified, the current value for [configoption] is output. If neither [configoption] nor [value] are specified, the whole current configuration is output. [configoption] must be of the format <section>.<option>"
 
-    def help_ls(self):
-        self.help_list()
-
     def help_passwd(self):
         self.usage("passwd")
         print "Changes the password on the database. "
@@ -669,7 +708,7 @@ the url must contain http:// or https://."
     def postloop(self):
         try:
             readline.write_history_file(self._historyfile)
-        except Exception, e:
+        except Exception:
             pass
 
     def __init__(self, db, hasxsel):
@@ -678,8 +717,8 @@ the url must contain http:// or https://."
         connecion, see if we have xsel ...
         """
         cmd.Cmd.__init__(self)
-        self.intro = "%s %s (c) %s <%s>" % (pwman.appname, pwman.version,
-                                            pwman.author, pwman.authoremail)
+        self.intro = "%s %s (c) visit: %s" % (pwman.appname, pwman.version,
+                                            pwman.website)
         self._historyfile = config.get_value("Readline", "history")
         self.hasxsel = hasxsel
         try:
@@ -693,7 +732,7 @@ the url must contain http:// or https://."
 
         try:
             readline.read_history_file(self._historyfile)
-        except Exception, e:
+        except IOError, e:
             pass
 
         self.prompt = "pwman> "
@@ -704,22 +743,22 @@ class PwmanCliMac(PwmanCli):
     """
     inherit from PwmanCli, override the right functions...
     """
-    def do_copy(self,args):
-        ids= self.get_ids(args)
+    def do_copy(self, args):
+        ids = self.get_ids(args)
         if len(ids) > 1:
             print "Can only 1 password at a time..."
         try:
             node = self._db.getnodes(ids)
             node[0].get_password()
             text_to_mcclipboard(node[0].get_password())
-            print """copied password for %s@%s clipboard... erasing in 10 sec...""" %\
+            print "copied password for %s@%s clipboard... erasing in 10 sec..." % \
             (node[0].get_username(), node[0].get_url())
             time.sleep(10)
             text_to_clipboards("")
         except Exception, e:
             self.error(e)
 
-    def do_cp(self,args):
+    def do_cp(self, args):
         self.do_copy(args)
 
     def do_open(self, args):
@@ -730,7 +769,7 @@ class PwmanCliMac(PwmanCli):
         try:
             node = self._db.getnodes(ids)
             url = node[0].get_url()
-            open_url(url,MacOS=True)
+            open_url(url, macosx=True)
         except Exception, e:
             self.error(e)
 
@@ -801,7 +840,10 @@ def getinput(question, default="", completer=None, width=_defaultwidth):
     if (not _readline_available):
         return raw_input(question.ljust(width))
     else:
-        def defaulter(): readline.insert_text(default)
+        def defaulter(): 
+            """define default behavior startup"""
+            readline.insert_text(default)
+
         readline.set_startup_hook(defaulter)
         oldcompleter = readline.get_completer()
         readline.set_completer(completer)
@@ -820,7 +862,7 @@ def getpassword(question, width=_defaultwidth, echo=False):
         while 1:
             a1 = getpass.getpass(question.ljust(width))
             if len(a1) == 0:
-                return a1;
+                return a1
             a2 = getpass.getpass("[Repeat] %s" % (question.ljust(width)))
             if a1 == a2:
                 return a1
@@ -832,11 +874,11 @@ def typeset(text, color, bold=False, underline=False):
     if not config.get_value("Global", "colors") == 'yes':
         return text
     if (bold):
-        bold = "%d;" %(ANSI.Bold)
+        bold = "%d ;" % (ANSI.Bold)
     else:
         bold = ""
     if (underline):
-        underline = "%d;" % (ANSI.Underline)
+        underline = "%d;" % (ANSI.Underscore)
     else:
         underline = ""
     return "\033[%s%s%sm%s\033[%sm" % (bold, underline, color,
@@ -846,9 +888,9 @@ def select(question, possible):
     for i in range(0, len(possible)):
         print ("%d - %-"+str(_defaultwidth)+"s") % (i+1, possible[i])
     while 1:
-        input = getonechar(question)
-        if input.isdigit() and int(input) in range(1, len(possible)+1):
-            return possible[int(input)-1]
+        uinput = getonechar(question)
+        if uinput.isdigit() and int(uinput) in range(1, len(possible)+1):
+            return possible[int(uinput)-1]
 
 def text_to_clipboards(text):
     """
@@ -868,7 +910,7 @@ def text_to_clipboards(text):
 please check your configuration file ... "
 
 
-def text_to_mcclipboards(text):
+def text_to_mcclipboard(text):
     """
     copy text to mac os x clip board
     credit:
@@ -882,15 +924,15 @@ def text_to_mcclipboards(text):
         print e, "\nExecuting pbcoy failed..."
 
 
-def open_url(link, MacOS=False):
+def open_url(link, macosx=False):
     """
     launch xdg-open or open in MacOSX with url
     """
-    uopen="xdg-open"
-    if MacOS:
-        uopen="open"
+    uopen = "xdg-open"
+    if macosx:
+        uopen = "open"
     try:
-       sp.Popen([uopen, link], stdin=sp.PIPE)
+        sp.Popen([uopen, link], stdin=sp.PIPE)
     except OSError, e:
         print "Executing open_url failed with:\n", e
 
@@ -923,11 +965,20 @@ class CliMenu(object):
             print "%c - Finish editing" % ('X')
             option = getonechar("Enter your choice:")
             try:
-                # substract 1 because array subscripts start at 1
+                # substract 1 because array subscripts start at 0
                 selection = int(option) - 1
-                value = self.items[selection].editor(self.items[selection].getter())
+                print "selection, ", selection
+                # new value is created by calling the editor with the 
+                # previous value as a parameter 
+                # TODO: enable overriding password policy as if new node
+                # is created.
+                if selection == 1: # for password
+                    value = self.items[selection].editor(0)
+                else:
+                    value = self.items[selection].editor(self.items[selection].getter())
+                
                 self.items[selection].setter(value)
-            except (ValueError,IndexError):
+            except (ValueError, IndexError):
                 if (option.upper() == 'X'):
                     break
                 print "Invalid selection"
